/*
 * Copyright (c) 2018 ARM Limited.
 *
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#ifndef __ARM_COMPUTE_GRAPH2_INODEVISITOR_H__
#define __ARM_COMPUTE_GRAPH2_INODEVISITOR_H__

#include "arm_compute/graph2/nodes/NodesFwd.h"

namespace arm_compute
{
namespace graph2
{
/**  Node visitor interface */
class INodeVisitor
{
public:
    virtual ~INodeVisitor()                              = default;
    virtual void visit(INode &n)                         = 0;
    virtual void visit(ActivationLayerNode &n)           = 0;
    virtual void visit(BatchNormalizationLayerNode &n)   = 0;
    virtual void visit(ConstNode &n)                     = 0;
    virtual void visit(ConvolutionLayerNode &n)          = 0;
    virtual void visit(DepthConcatenateLayerNode &n)     = 0;
    virtual void visit(DepthwiseConvolutionLayerNode &n) = 0;
    virtual void visit(EltwiseLayerNode &n)              = 0;
    virtual void visit(FlattenLayerNode &n)              = 0;
    virtual void visit(FullyConnectedLayerNode &n)       = 0;
    virtual void visit(InputNode &n)                     = 0;
    virtual void visit(NormalizationLayerNode &n)        = 0;
    virtual void visit(OutputNode &n)                    = 0;
    virtual void visit(PoolingLayerNode &n)              = 0;
    virtual void visit(ReshapeLayerNode &n)              = 0;
    virtual void visit(SoftmaxLayerNode &n)              = 0;
};

/** Default visitor implementation
 *
 * Implements visit methods by calling a default function.
 * Inherit from DefaultNodeVisitor if you don't want to provide specific implementation for all nodes.
 */
class DefaultNodeVisitor : public INodeVisitor
{
public:
    virtual ~DefaultNodeVisitor() = default;

    // Inherited methods overridden
    virtual void visit(INode &n) override
    {
        default_visit();
    }
    virtual void visit(ActivationLayerNode &n) override
    {
        default_visit();
    }
    virtual void visit(BatchNormalizationLayerNode &n) override
    {
        default_visit();
    }
    virtual void visit(ConstNode &n) override
    {
        default_visit();
    }
    virtual void visit(ConvolutionLayerNode &n) override
    {
        default_visit();
    }
    virtual void visit(DepthConcatenateLayerNode &n) override
    {
        default_visit();
    }
    virtual void visit(DepthwiseConvolutionLayerNode &n) override
    {
        default_visit();
    }
    virtual void visit(EltwiseLayerNode &n) override
    {
        default_visit();
    }
    virtual void visit(FlattenLayerNode &n) override
    {
        default_visit();
    }
    virtual void visit(FullyConnectedLayerNode &n) override
    {
        default_visit();
    }
    virtual void visit(InputNode &n) override
    {
        default_visit();
    }
    virtual void visit(NormalizationLayerNode &n) override
    {
        default_visit();
    }
    virtual void visit(OutputNode &n) override
    {
        default_visit();
    }
    virtual void visit(PoolingLayerNode &n) override
    {
        default_visit();
    }
    virtual void visit(ReshapeLayerNode &n) override
    {
        default_visit();
    }
    virtual void visit(SoftmaxLayerNode &n) override
    {
        default_visit();
    }

    /** Function to be overloaded by the client and implement default behavior for the
     *  non-overloaded visitors
     */
    virtual void default_visit() = 0;
};
} // namespace graph2
} // namespace arm_compute
#endif /* __ARM_COMPUTE_GRAPH2_INODEVISITOR_H__ */
